/******************************************************************************
*  One spell in the Wizard Spell Compendium database.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2017-06-23
*  @version  1.03
******************************************************************************/

public class Spell {

	//--------------------------------------------------------------------------
	//  Constants
	//--------------------------------------------------------------------------

	static final char NULL_FREQUENCY = '-';

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	int level;
	String name, reversal, origin, schools;
	String range, components, materials, castTime, duration, area, save;
	char frequency;
	int volume, page;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**	
	*  Constructor.
	*/
	Spell (String[] spec) {
		level = Integer.parseInt(spec[0]);
		name = spec[1];
		reversal = spec[2];
		origin = spec[3];
		schools = spec[4];
		range = spec[5];		
		components = spec[6];
		materials = spec[7];
		castTime = spec[8];
		duration = spec[9];
		area = spec[10];
		save = spec[11];
		frequencyFromString(spec[12]);
		volume = Integer.parseInt(spec[13]);
		page = Integer.parseInt(spec[14]);
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**	
	*  Get name.
	*/
	public String getName () {
		return name;
	}

	/**	
	*  Does this spell have a reversed version?
	*/
	public boolean hasReversal () {
		return reversal.length() > 0;	
	}
	
	/**	
	*  Get the name of the the reversed spell.
	*/
	public String getReversal () {
		return reversal;
	}

	/**	
	*  Does this spell include a given school?
	*/
	public boolean hasSchool (String s) {
		return schools.contains(s);	
	}

	/**	
	*  Get school specifiers.
	*/
	public String getSchools () {
		return schools;
	}

	/**	
	*  Does this spell have an origin specification?
	*/
	public boolean hasOrigin () {
		return origin.length() > 0;
	}

	/**	
	*  Get the origin specification.
	*/
	public String getOrigin () {
		return origin;
	}

	/**
	*  Set frequency from string.
	*/
	void frequencyFromString (String s) {
		if (s == null || s.length() == 0)
			frequency = NULL_FREQUENCY;
		else
			frequency = s.charAt(0);
	}

	/**
	*  Convert frequency to string.
	*/
	String frequencyToString () {
		if (frequency == NULL_FREQUENCY)
			return "";
		else
			return Character.toString(frequency);	
	}

	/**
	*  Identify this object as a string.
	*/
	public String toString () {
		return name;
	}

	/**
	*  Identify this object as line in CSV file.
	*/
	public String toCSVString() {
		return CSVReader.makeLineFromStrings(
			Integer.toString(level), name, reversal, origin, schools,
			range, components, materials, castTime, duration, area, save,
			frequencyToString(), Integer.toString(volume), 
			Integer.toString(page));
	}

	/**
	*  Create header line for CSV file.
	*/
	static public String getCSVHeader () {
		String s = CSVReader.makeLineFromStrings(
			"Level", "Name", "Reversal", "Origin", "Schools", "Range", 
			"Components", "Materials", "Cast Time", "Duration", "Area", "Save",
			"Frequency", "Volume", "Page");
		return s;	
	}

	/**
	*  Main test function.
	*/
	public static void main (String[] args) {
		String[] spec = {"1", "Affect Normal Fires", "", "", "Alteration", 
			"5 yds/level", "VSM", "A fire source", "1", "2 rnds/level", 
			"10 ft rad", "None", "C", "1", "25"};
		Spell s = new Spell(spec);
		System.out.println(s);
	}
}

